;;; Directory Local Variables
;;; For more information see (info "(emacs) Directory Variables")

((nil . ((projectile-project-compilation-cmd . "source .envrc; make format")))
  (python-mode . ((fill-column . 100))))
